#include <stdio.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <string.h>
#include <errno.h>

int main(int argc, char *argv[])
{
  pid_t p;

  if ((p = fork()) == -1) {
    perror(":(");
    return 1;
  } else if (p == 0) {

    // child code

    // Ignore SIGINT.
    struct sigaction a;
    a.sa_handler = SIG_IGN;
    a.sa_flags = 0;
    sigaction(SIGINT, &a, NULL);

    int r, e;

    // 2. Wait for any signal using pause. Three possibilities:
    // a. If killed by signal: this code won't even know :)
    // b. If signal ignored: pause keeps waiting.
    // c. If signal handled: pause returns -1, errno = EINTR
    r = pause();
    e = errno;
    printf("pause syscall returns %d, errno=%d: %s\n", r, e, strerror(e));

    // If pause returns, I want to do normal exit.
    return 0;

  } else {

    // parent code

    printf("child PID=%d\n", p);
    printf("I will wait indefintely for it to die.\n");
    printf("If it doesn't, you can go to another terminal and use: ");
    printf("kill %d\n", p);

    // Delay a bit to let child reach "pause", then send SIGINT.
    sleep(1);
    kill(p, SIGINT);

    // Now wait and report.
    int s;
    wait(&s);
    if (WIFEXITED(s)) {
      printf("child normal exit code: %d\n", WEXITSTATUS(s));
    } else if (WIFSIGNALED(s)) {
      printf("child died by signal %s(%d)\n", strsignal(WTERMSIG(s)), WTERMSIG(s));
    }
    return 0;

  }
}
